#!/bin/bash

# Any input file should exist with the same name in train, tune, and test subdirs
INPUTDIR=$PWD/testInput
TRAINDIR=$INPUTDIR/train
TUNEDIR=$INPUTDIR/tune
TESTDIR=$INPUTDIR/test
OUTDIR=$INPUTDIR/out
REFDIR=$INPUTDIR/sampleOut

# Verify that the necessary files are there and that 
test -e HW2.class || make
test -e DecisionTreeImpl.class || make
test DecisionTreeImpl.class -nt DecisionTreeImpl.java 
if [ $? -ne 0 ] ; then
    REPLY=""
    while [ "$REPLY" = "" ] ; do
        read -r -p "Your DecisionTreeImpl.java is newer than its class.  Remake? [Y/n]" REPLY
        # This is really a make-or-break decision, if you get my drift
        case $response in
            [yY])
                make
                ;;
            [nN])
                break
                ;;
            *)
                REPLY=""
                ;;
        esac
    done
fi

for file in `ls $PWD/testInput/train`; do

    # Verify that all necessary files exist
    test -e $TUNEDIR/$file || continue
    test -e $TESTDIR/$file || continue

    echo "Running your code using input: $file"

    echo "" > $OUTDIR/$file
    for mode in {0..4}; do
        echo "Mode=$mode" >> $OUTDIR/$file
        java HW2 $mode $TRAINDIR/$file $TUNEDIR/$file $TESTDIR/$file >> $OUTDIR/$file
        echo -e "\n\n" >> $OUTDIR/$file
    done

    test -e $REFDIR/$file || continue
    echo "Testing if your output matches $REFDIR/$file"
    diff $REFDIR/$file $OUTDIR/$file && echo -e "\t Success" || echo -e "\t Failure"
done

echo "Checking if you modified any files that shouldn't have been modified"
bad="0"
for file in `ls $INPUTDIR/origSrc/*.java`; do
    diff $file `basename $file`|| bad="1"
done

if [ "$bad" = "1" ] ; then
    echo -e "\t WARNING:  Your files may be different than what they want"
else
    echo -e "\t All files look good"
fi
